'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/* eslint-env browser */

var _require = require('redux'),
    createStore = _require.createStore,
    compose = _require.compose,
    applyMiddleware = _require.applyMiddleware,
    combineReducers = _require.combineReducers;

var createSagaMiddleware = require('redux-saga');

var _require2 = require('redux-logger'),
    createLogger = _require2.createLogger;

var logger = createLogger();

/**
 * 创建store
 *
 * 通过webpack替换获取到所有的reducer文件，再通过combineReducers组装到一起
 * 创建saga中间件，将中间件挂载到store上
 * 返回store
 *
 * @param    {object} state
 * @returns  {object} store
 *
 * @date     2018-03-01
 * @author   zhangyang<zhangyang@zhongan.com>
 */
module.exports = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var composeEnhancers = (typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object' && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ? window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ : compose;
  var reducers = {};
  var commonReducers = void 0;
  var reducerFilePaths = global.__CLIENT_REDUCER__;
  var commonReducerFilePaths = global.__COMMON_CLIENT_REDUCER__;

  Object.keys(reducerFilePaths).forEach(function (i) {
    reducers[i] = require(global.__RELATIVE_PATH__ + '/' + reducerFilePaths[i]);
  });

  Object.keys(commonReducerFilePaths).forEach(function (i) {
    commonReducers = require(global.__COMMON_RELATIVE_PATH__ + '/' + commonReducerFilePaths[i]);
  });

  // combineReducers does not accept void object, so switch void reducers to a function
  var moduleReducer = Object.keys(reducers).length ? combineReducers(reducers) : function (moduleState) {
    return moduleState || {};
  };

  // 扩展实际项目中的common reducer, 通过该入口能够以键值对的形式嵌入多个与common同级的redux strore key, 默认只有common
  var funseeReducer = typeof commonReducers === 'function' ? {
    module: moduleReducer,
    common: commonReducers
  } : _extends({
    module: moduleReducer
  }, commonReducers);

  // Q: default ?
  var sagaMiddleware = createSagaMiddleware.default();

  var store = createStore(combineReducers(funseeReducer), state, composeEnhancers(applyMiddleware(sagaMiddleware, logger)));

  store.runSaga = sagaMiddleware.run;
  store.close = function () {
    return store.dispatch(createSagaMiddleware.END);
  };

  return store;
};